"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const yeoman_test_1 = __importDefault(require("yeoman-test"));
const RunResult_utils_1 = require("./RunResult.utils");
const testGenerators_1 = require("./testGenerators");
describe("RunResultUtils", () => {
    let result;
    describe("when package.json doesn't exist", () => {
        beforeAll(async () => {
            result = await yeoman_test_1.default.create(testGenerators_1.EmptyTestGenerator).run();
        });
        describe(".readPackageJson", () => {
            it("throws an error when package.json doesn't exist", () => {
                expect(() => {
                    RunResult_utils_1.RunResultUtils.readPackageJson(result);
                }).toThrow("package.json doesn't exist");
            });
        });
    });
    describe("when package.json exists", () => {
        beforeAll(async () => {
            result = await yeoman_test_1.default.create(testGenerators_1.DevDependenciesTestGenerator).run();
        });
        describe(".readPackageJson", () => {
            it("returns the contents of package.json", () => {
                const json = RunResult_utils_1.RunResultUtils.readPackageJson(result);
                expect(json).toEqual({
                    devDependencies: {
                        typescript: "4.7.4",
                    },
                    devDependenciesComments: {
                        typescript: "TypeScript support for this project",
                    },
                });
            });
        });
        describe(".write", () => {
            afterEach(async () => {
                await RunResult_utils_1.RunResultUtils.delete(result, "test.txt");
            });
            it("writes the provided contents to the provided path", async () => {
                await RunResult_utils_1.RunResultUtils.write(result, "test.txt", "Hello, world!");
                result.assertFileContent("test.txt", "Hello, world!");
            });
        });
        describe(".delete", () => {
            it("deletes the specified file", async () => {
                await RunResult_utils_1.RunResultUtils.write(result, "test.txt", "Hello, world!");
                result.assertFile("test.txt");
                await RunResult_utils_1.RunResultUtils.delete(result, "test.txt");
                result.assertNoFile("test.txt");
            });
        });
        describe(".gitInit", () => {
            beforeAll(() => {
                RunResult_utils_1.RunResultUtils.gitInit(result);
            });
            it("initializes an new git repository", () => {
                expect(() => {
                    result.env.spawnCommandSync("git", ["status"], {});
                }).not.toThrow();
            });
        });
        describe(".gitConfigUser", () => {
            describe("sets the user for the git repository", () => {
                it("sets the user's email address", () => {
                    const commandResult = result.env.spawnCommandSync("git", ["config", "user.email"], { stdio: ["ignore", "pipe", "pipe"] });
                    const gitUserEmail = commandResult.stdout;
                    expect(gitUserEmail).toBe("testing@jest.io");
                });
                it("sets the user's name", () => {
                    const commandResult = result.env.spawnCommandSync("git", ["config", "user.name"], { stdio: ["ignore", "pipe", "pipe"] });
                    const gitUserEmail = commandResult.stdout;
                    expect(gitUserEmail).toBe("Jest");
                });
            });
        });
        describe(".gitRestoreStaged", () => {
            it("clears all staged files for the git repository", () => {
                result.env.spawnCommandSync("git", ["commit", "--allow-empty", "-m", "Initial commit"], {});
                result.env.spawnCommandSync("git", ["add", "package.json"], {});
                RunResult_utils_1.RunResultUtils.gitRestoreStaged(result);
                const commandResult = result.env.spawnCommandSync("git", ["diff", "--staged", "--quiet"], { stdio: ["ignore", "pipe", "pipe"] });
                const gitStatus = commandResult.stdout;
                expect(gitStatus).toBe("");
            });
        });
    });
});
//# sourceMappingURL=RunResult.utils.test.js.map