"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const yeoman_generator_1 = __importDefault(require("yeoman-generator"));
const yeoman_test_1 = __importDefault(require("yeoman-test"));
const __utils__1 = require("../__tests__/__utils__");
const __utils__2 = require("../__tests__/__utils__");
describe("BaseGenerator", () => {
    let result;
    describe("#loadConfig", () => {
        beforeAll(async () => {
            result = await yeoman_test_1.default.create(__utils__1.LoadConfigTestGenerator).run();
        });
        it("loads generator prompt answers from a provided YML file", () => {
            result.assertJsonFileContent("answers.json", {
                packageName: "load-config-test",
                packageDescription: "Loaded description from config",
            });
        });
    });
    describe("#composeWithSubGenerator", () => {
        describe("subgenerator", () => {
            describe("when parent is given options", () => {
                beforeAll(async () => {
                    result = await yeoman_test_1.default.create(__utils__2.ComposeWithSubGeneratorTestGenerator, {
                        namespace: "test:compose-with-sub-generator",
                    })
                        .withOptions({
                        packageName: "test-package",
                        packageDescription: "Test package description",
                        packageKeywords: "test foo bar",
                        yarnInstall: true,
                    })
                        .run();
                });
                describe("inherits answers from the parent", () => {
                    it("writes the child answers to inheritedAnswers.json", () => {
                        result.assertJsonFileContent("inheritedAnswers.json", {
                            packageName: "test-package",
                            packageDescription: "Test package description",
                        });
                    });
                    it("writes parent answers to parentAnswers.json", () => {
                        result.assertJsonFileContent("parentAnswers.json", {
                            packageName: "test-package",
                            packageDescription: "Test package description",
                            packageKeywords: "test foo bar",
                        });
                    });
                });
                describe("inherits options from the parent", () => {
                    it("writes the child options to inheritedOptions.json", () => {
                        result.assertJsonFileContent("inheritedOptions.json", {
                            packageName: "test-package",
                            packageDescription: "Test package description",
                            packageKeywords: "test foo bar",
                            yarnInstall: true,
                        });
                    });
                    it("writes the parent options to inheritedOptions.json", () => {
                        result.assertJsonFileContent("parentOptions.json", {
                            packageName: "test-package",
                            packageDescription: "Test package description",
                            packageKeywords: "test foo bar",
                            yarnInstall: true,
                        });
                    });
                });
            });
            describe("when parent is given prompt answers", () => {
                let promptSpy;
                beforeAll(async () => {
                    promptSpy = jest.spyOn(yeoman_generator_1.default.prototype, "prompt");
                    result = await yeoman_test_1.default.create(__utils__2.ComposeWithSubGeneratorTestGenerator, {
                        namespace: "test:compose-with-sub-generator",
                    })
                        .withPrompts({
                        packageName: "test-package",
                        packageDescription: "Test package description",
                        packageKeywords: "test foo bar",
                    })
                        .withOptions({
                        yarnInstall: true,
                    })
                        .run();
                });
                it("writes the child answers to inheritedAnswers.json", () => {
                    result.assertJsonFileContent("inheritedAnswers.json", {
                        packageName: "test-package",
                        packageDescription: "Test package description",
                    });
                });
                it("writes parent answers to parentAnswers.json", () => {
                    result.assertJsonFileContent("parentAnswers.json", {
                        packageName: "test-package",
                        packageDescription: "Test package description",
                        packageKeywords: "test foo bar",
                    });
                });
                it("still inherits options", () => {
                    result.assertJsonFileContent("inheritedOptions.json", {
                        yarnInstall: true,
                    });
                });
                it("does not double prompt", () => {
                    expect(promptSpy).toHaveBeenCalledTimes(1);
                });
            });
        });
    });
    describe("interchangeably accepts options or prompt answers", () => {
        describe("when given options", () => {
            beforeAll(async () => {
                result = await yeoman_test_1.default.create(__utils__1.OptionsTestGenerator, {
                    namespace: "test:options",
                })
                    .withOptions({ name: "test", description: "a simple test" })
                    .run();
            });
            describe("writes options to options.json", () => {
                it("creates an options.json file", () => {
                    result.assertFile("options.json");
                });
                it("writes all options to options.json", () => {
                    result.assertJsonFileContent("options.json", {
                        name: "test",
                        description: "a simple test",
                    });
                });
            });
        });
        describe("when given prompt answers", () => {
            beforeAll(async () => {
                result = await yeoman_test_1.default.create(__utils__1.OptionsTestGenerator, {
                    namespace: "test:options",
                })
                    .withPrompts({ name: "test", description: "a simple test" })
                    .run();
            });
            describe("writes answers to options.json", () => {
                it("creates an options.json file", () => {
                    result.assertFile("options.json");
                });
                it("writes all answers to options.json", () => {
                    result.assertJsonFileContent("options.json", {
                        name: "test",
                        description: "a simple test",
                    });
                });
            });
        });
        describe("when given a mix of options and prompt answers", () => {
            beforeAll(async () => {
                result = await yeoman_test_1.default.create(__utils__1.OptionsTestGenerator, {
                    namespace: "test:options",
                })
                    .withOptions({ name: "test" })
                    .withPrompts({ description: "a simple test" })
                    .run();
            });
            describe("writes options and answers to options.json", () => {
                it("creates an options.json file", () => {
                    result.assertFile("options.json");
                });
                it("writes all options and answers to options.json", () => {
                    result.assertJsonFileContent("options.json", {
                        name: "test",
                        description: "a simple test",
                    });
                });
            });
        });
    });
});
//# sourceMappingURL=BaseGenerator.test.js.map